@@ -81,22 +81,12 @@ public:
      * Defaults to true.
      */
     void setLeadingWhitespace(bool enable);
-    /**
-     * Returns whether leading whitespace at the end of lines is included
-     * in the output.
-     */
-    bool leadingWhitespace() const;
     /**
      * Set whether trailing whitespace at the end of lines should be included
      * in the output.
      * Defaults to true.
      */
     void setTrailingWhitespace(bool enable);
-    /**
-     * Returns whether trailing whitespace at the end of lines is included
-     * in the output.
-     */
-    bool trailingWhitespace() const;
     /**
      * Returns of character positions in the output stream
      * at which new lines where added.  Returns an empty if setTrackLinePositions() is false or if
